@@ -30,7 +30,6 @@
 #include "eog-thumb-view.h"
 
 #if HAVE_EXIF
-#include "eog-exif-details.h"
 #include "eog-exif-util.h"
 #endif
 
@@ -46,11 +45,14 @@
 #include <exempi/xmp.h>
 #include <exempi/xmpconsts.h>
 #endif
-
 #if HAVE_EXIF || HAVE_EXEMPI
 #define HAVE_METADATA 1
 #endif
 
+#if HAVE_METADATA
+#include "eog-exif-details.h"
+#endif
+
 #define EOG_PROPERTIES_DIALOG_GET_PRIVATE(object) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOG_TYPE_PROPERTIES_DIALOG, EogPropertiesDialogPrivate))
 
@@ -83,7 +85,6 @@ struct _EogPropertiesDialogPrivate {
 	GtkWidget      *created_label;
 	GtkWidget      *modified_label;
 #ifdef HAVE_EXIF
-	GtkWidget      *exif_box;
 	GtkWidget      *exif_aperture_label;
 	GtkWidget      *exif_exposure_label;
 	GtkWidget      *exif_focal_label;
@@ -102,6 +103,7 @@ struct _EogPropertiesDialogPrivate {
 	GtkWidget      *xmp_details;
 #endif
 #if HAVE_METADATA
+	GtkWidget      *exif_box;
 	GtkWidget      *exif_details_expander;
 	GtkWidget      *exif_details;
 #endif
@@ -160,6 +162,29 @@ pd_update_general_tab (EogPropertiesDialog *prop_dlg,
 	g_free (dir_str);
 }
 
+#if HAVE_EXIF
+static void
+eog_exif_set_label (GtkWidget *w, ExifData *exif_data, gint tag_id)
+{
+	gchar exif_buffer[512];
+	const gchar *buf_ptr;
+	gchar *label_text = NULL;
+
+	if (exif_data) {
+		buf_ptr = eog_exif_util_get_value (exif_data, tag_id,
+						   exif_buffer, 512);
+		
+		if (tag_id == EXIF_TAG_DATE_TIME_ORIGINAL && buf_ptr)
+			label_text = eog_exif_util_format_date (buf_ptr);
+		else
+			label_text = eog_util_make_valid_utf8 (buf_ptr);
+	}
+
+	gtk_label_set_text (GTK_LABEL (w), label_text);
+	g_free (label_text);
+}
+#endif
+
 #if HAVE_EXEMPI
 static void
 eog_xmp_set_label (XmpPtr xmp, 
@@ -203,7 +228,12 @@ eog_xmp_set_label (XmpPtr xmp,
 			}
 
 			gtk_label_set_text (GTK_LABEL (w), string->str);
+			g_string_free (string, TRUE);
 		}
+	} else {
+		/* Property was not found */
+		/* Clear label so it won't show bogus data */
+		gtk_label_set_text (GTK_LABEL (w), NULL);
 	}
 
 	xmp_string_free (value);
@@ -219,7 +249,6 @@ pd_update_metadata_tab (EogPropertiesDialog *prop_dlg,
 	GtkNotebook *notebook;
 #if HAVE_EXIF
 	ExifData    *exif_data;
-	const gchar *exif_value = NULL;
 #endif
 #if HAVE_EXEMPI
 	XmpPtr      xmp_data;
@@ -231,7 +260,10 @@ pd_update_metadata_tab (EogPropertiesDialog *prop_dlg,
 
 	notebook = GTK_NOTEBOOK (priv->notebook);
 
-	if (!eog_image_has_data (image, EOG_IMAGE_DATA_EXIF)
+	if (TRUE
+#if HAVE_EXIF
+	    && !eog_image_has_data (image, EOG_IMAGE_DATA_EXIF)
+#endif
 #if HAVE_EXEMPI
 	    && !eog_image_has_data (image, EOG_IMAGE_DATA_XMP)
 #endif
@@ -252,52 +284,39 @@ pd_update_metadata_tab (EogPropertiesDialog *prop_dlg,
 #if HAVE_EXIF
 	exif_data = (ExifData *) eog_image_get_exif_info (image);
 
-	if (exif_data) {
-		exif_value = eog_exif_util_get_value (exif_data,
-						      EXIF_TAG_FNUMBER);
-		gtk_label_set_text (GTK_LABEL (priv->exif_aperture_label), 
-				    eog_util_make_valid_utf8 (exif_value));
+	eog_exif_set_label (priv->exif_aperture_label,
+			    exif_data, EXIF_TAG_FNUMBER);
 
-		exif_value = eog_exif_util_get_value (exif_data,
-						      EXIF_TAG_EXPOSURE_TIME);
-		gtk_label_set_text (GTK_LABEL (priv->exif_exposure_label),
-				    eog_util_make_valid_utf8 (exif_value));
+	eog_exif_set_label (priv->exif_exposure_label,
+			    exif_data, EXIF_TAG_EXPOSURE_TIME);
 
-		exif_value = eog_exif_util_get_value (exif_data,
-						      EXIF_TAG_FOCAL_LENGTH);
-		gtk_label_set_text (GTK_LABEL (priv->exif_focal_label), 
-				    eog_util_make_valid_utf8 (exif_value));
+	eog_exif_set_label (priv->exif_focal_label, 
+			    exif_data, EXIF_TAG_FOCAL_LENGTH);
 
-		exif_value = eog_exif_util_get_value (exif_data,
-						      EXIF_TAG_FLASH);
-		gtk_label_set_text (GTK_LABEL (priv->exif_flash_label),
-				    eog_util_make_valid_utf8 (exif_value));
+	eog_exif_set_label (priv->exif_focal_label, 
+			    exif_data, EXIF_TAG_FOCAL_LENGTH);
 
-		exif_value = eog_exif_util_get_value (exif_data,
-						      EXIF_TAG_ISO_SPEED_RATINGS);
-		gtk_label_set_text (GTK_LABEL (priv->exif_iso_label), 
-				    eog_util_make_valid_utf8 (exif_value));
+	eog_exif_set_label (priv->exif_flash_label,
+			    exif_data, EXIF_TAG_FLASH);
 
-		exif_value = eog_exif_util_get_value (exif_data,
-						      EXIF_TAG_METERING_MODE);
-		gtk_label_set_text (GTK_LABEL (priv->exif_metering_label),
-				    eog_util_make_valid_utf8 (exif_value));
+	eog_exif_set_label (priv->exif_iso_label, 
+			    exif_data, EXIF_TAG_ISO_SPEED_RATINGS);
 
-		exif_value = eog_exif_util_get_value (exif_data,
-						      EXIF_TAG_MODEL);
-		gtk_label_set_text (GTK_LABEL (priv->exif_model_label),
-				    eog_util_make_valid_utf8 (exif_value));
 
-		exif_value = eog_exif_util_get_value (exif_data,
-						      EXIF_TAG_DATE_TIME);
-		gtk_label_set_text (GTK_LABEL (priv->exif_date_label),
-				    eog_exif_util_format_date (exif_value));
+	eog_exif_set_label (priv->exif_metering_label,
+			    exif_data, EXIF_TAG_METERING_MODE);
 
-		eog_exif_details_update (EOG_EXIF_DETAILS (priv->exif_details), 
-					 exif_data);
+	eog_exif_set_label (priv->exif_model_label,
+			    exif_data, EXIF_TAG_MODEL);
 
-		exif_data_unref(exif_data);
-	}
+	eog_exif_set_label (priv->exif_date_label,
+			    exif_data, EXIF_TAG_DATE_TIME_ORIGINAL);
+
+	eog_exif_details_update (EOG_EXIF_DETAILS (priv->exif_details), 
+				 exif_data);
+
+	/* exif_data_unref can handle NULL-values */
+	exif_data_unref(exif_data);
 #endif
 
 #if HAVE_EXEMPI
@@ -332,6 +351,17 @@ pd_update_metadata_tab (EogPropertiesDialog *prop_dlg,
 		eog_exif_details_xmp_update (EOG_EXIF_DETAILS (priv->exif_details), xmp_data);
 
 		xmp_free (xmp_data);
+	} else {
+		/* Image has no XMP data */
+
+		/* Clear the labels so they won't display foreign data.*/
+
+		gtk_label_set_text (GTK_LABEL (priv->xmp_location_label), NULL);
+		gtk_label_set_text (GTK_LABEL (priv->xmp_description_label),
+				    NULL);
+		gtk_label_set_text (GTK_LABEL (priv->xmp_keywords_label), NULL);
+		gtk_label_set_text (GTK_LABEL (priv->xmp_creator_label), NULL);
+		gtk_label_set_text (GTK_LABEL (priv->xmp_rights_label), NULL);
 	}
 #endif
 }
@@ -511,7 +541,6 @@ eog_properties_dialog_init (EogPropertiesDialog *prop_dlg)
 			         "created_label", &priv->created_label,
 			         "modified_label", &priv->modified_label,
 #ifdef HAVE_EXIF
-			         "exif_box", &priv->exif_box,
 			         "exif_aperture_label", &priv->exif_aperture_label,
 			         "exif_exposure_label", &priv->exif_exposure_label,
 			         "exif_focal_label", &priv->exif_focal_label,
@@ -520,7 +549,6 @@ eog_properties_dialog_init (EogPropertiesDialog *prop_dlg)
 			         "exif_metering_label", &priv->exif_metering_label,
 			         "exif_model_label", &priv->exif_model_label,
 			         "exif_date_label", &priv->exif_date_label,
-			         "exif_details_expander", &priv->exif_details_expander,
 #endif
 #ifdef HAVE_EXEMPI
 				 "xmp_location_label", &priv->xmp_location_label,
@@ -532,7 +560,10 @@ eog_properties_dialog_init (EogPropertiesDialog *prop_dlg)
 				 "xmp_box", &xmp_box,
 				 "xmp_box_label", &xmp_box_label,
 #endif
-
+#ifdef HAVE_METADATA
+			         "exif_box", &priv->exif_box,
+				 "exif_details_expander", &priv->exif_details_expander,
+#endif
 			         NULL);
 
 	g_signal_connect (dlg,
